from abc import ABCMeta
from contextvars import ContextVar, Token
from dataclasses import dataclass, field
from types import TracebackType
from typing import Any, Optional

from typing_extensions import Self, dataclass_transform


@dataclass_transform(
    eq_default=True,
    order_default=False,
    kw_only_default=True,
    field_descriptors=(field,),
)
class MessageMeta(ABCMeta):
    def __new__(
        cls: type[Self], name: str, bases: tuple[type, ...], namespace: dict[str, Any]
    ) -> Self:
        new_cls = super().__new__(cls, name, bases, namespace)
        return dataclass(frozen=True, kw_only=True)(new_cls)  # type: ignore


class Message(metaclass=MessageMeta):
    ...


def issue(message: Message):
    messages = messages_context_var.get()
    messages.add(message)


messages_context_var: ContextVar[set[Message]] = ContextVar("messages")


@dataclass(slots=True)
class MessageCatchContext:

    messages: set[Message] = field(default_factory=set, init=False)
    _token: Token[set[Message]] = field(init=False)

    def __enter__(self):
        self._token = messages_context_var.set(set())
        return self

    def __exit__(
        self,
        __exc_type: Optional[type[BaseException]],
        __exc_value: Optional[BaseException],
        __traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        self.messages = messages_context_var.get()
        messages_context_var.reset(self._token)



from abc import ABCMeta, abstractmethod
from dataclasses import dataclass, field
from typing import Any

from typing_extensions import Self, dataclass_transform


@dataclass_transform(
    eq_default=True,
    order_default=False,
    kw_only_default=True,
    field_descriptors=(field,),
)
class EntityMeta(ABCMeta):
    def __new__(
        cls: type[Self], name: str, bases: tuple[type, ...], namespace: dict[str, Any]
    ) -> Self:
        new_cls = super().__new__(cls, name, bases, namespace)
        return dataclass(eq=False, kw_only=True)(new_cls)  # type: ignore


class Entity(metaclass=EntityMeta):
    @property
    @abstractmethod
    def identity(self) -> Any:
        ...



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pydamain',
 'pydamain.adapter',
 'pydamain.adapter.in_',
 'pydamain.adapter.out_',
 'pydamain.adapter.out_.repository',
 'pydamain.adapter.out_.unit_of_work',
 'pydamain.domain',
 'pydamain.domain.messages',
 'pydamain.domain.models',
 'pydamain.domain.service',
 'pydamain.port',
 'pydamain.port.in_',
 'pydamain.port.out_']

package_data = \
{'': ['*']}

install_requires = \
['typing-extensions>=4.2.0,<5.0.0']

extras_require = \
{'aiosmtplib': ['aiosmtplib>=1.1.6,<2.0.0'],
 'sqlalchemy': ['SQLAlchemy>=1.4.36,<2.0.0']}

setup_kwargs = {
    'name': 'pydamain',
    'version': '0.4.5',
    'description': '도메인 주도 설계 라이브러리.',
    'long_description': '# Pydamain\n\n작성중...',
    'author': 'Jongbeom-Kwon',
    'author_email': 'bolk9652@naver.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/by-Exist/pydamain',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
